% Ordinary Differential Equations - Boundary Problem - Difference Method
% y''+2y'-3y=0, y(xa)=0; y(xb)=1 for xa=0, xb=4
%%%%%% A. Symbolic Solution %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  clear; delete(get(0,'children'))
  xa=0; xb=4;
  ys=dsolve('D2y+2*Dy-3*y=0','y(0)=0','y(4)=1','x');
    pretty(simplify(ys)); 
    ezplot(ys,[xa xb]); grid on; axis tight
%%%%%% B. Numeric Solution: Difference Method %%%%%%%%%%%%
% Equation: y''(k)+2*y(k)+3*y(k); y(xa)=0; y(xb)=1;
%     h=(xb-xa)/N;
%     1/h^2(y(k-1)-2y(k)+y(k+1))+2*1/(2h)*(y(k+1)-y(k-1))-3*y(k)=0
  N=4; h=(xb-xa)/N; x=xa:h:xb;
  ya=0; yb=1;
  A=[(-2/h^2-3)   (1/h^2+1/h)    0
     (1/h^2-1/h) (-2/h^2-3)  (1/h^2+1/h) 
          0     (1/h^2-1/h) (-2/h^2-3)  ]
  b= [-(1/h^2-1/h)*ya  0  -(1/h^2+1/h)*yb]'
  y=inv(A)*b; y=[ya; y; yb];
  hold on; t=plot(x,y,'or'); axis tight; hold off
  v=axis;
    t1=text(v(1)+0.4*(v(2)-v(1)), v(3)+0.62*(v(4)-v(3)),'DIFFERENCE METHOD');
    t2=text(v(1)+0.52*(v(2)-v(1)), v(3)+0.57*(v(4)-v(3)),['N= ',num2str(N)]);
% General Solution %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
  k=input('General solution (y|n): ','s');  
    if k=='n'; break, end
  while 1==1
    N=input('Number of samples (>2 or <0 to stop) ...: ');
    if N<0, break, end
    delete(t); delete(t1); delete(t2)
      h=(xb-xa)/N; x=[xa:h:xb]';
      ya=0; yb=1;
    A=diag(ones(1,N-1)*(-2/h^2-3))+...
      diag(ones(1,N-2)*(1/h^2+1/h),1)+...
      diag(ones(1,N-2)*(1/h^2-1/h),-1) 
    b=[-(1/h^2-1/h)*ya  zeros(1,N-3) -(1/h^2+1/h)*yb]'
    y=inv(A)*b; y=[ya; y; yb];
    hold on; t=plot(x,y,'or'); axis tight; hold off
    v=axis;
     t1=text(v(1)+0.4*(v(2)-v(1)), v(3)+0.62*(v(4)-v(3)),'DIFFERENCE METHOD');
     t2=text(v(1)+0.52*(v(2)-v(1)), v(3)+0.57*(v(4)-v(3)),['N= ',num2str(N)]);
  end